@@ -16,6 +16,7 @@
 import logging
 import time
 import urllib2, urllib
+import re
 import xml.etree.ElementTree as etree
 
 from pithos.pandora.xmlrpc import *
@@ -34,9 +35,10 @@ RATE_NONE = None
 PLAYLIST_VALIDITY_TIME = 60*60*3
 
 class PandoraError(IOError):
-    def __init__(self, message, status=None):
+    def __init__(self, message, status=None, submsg=None):
         self.status = status
         self.message = message
+        self.submsg = submsg
         
 class PandoraAuthTokenInvalid(PandoraError): pass
 class PandoraNetError(PandoraError): pass
@@ -75,12 +77,15 @@ class Pandora(object):
         self.set_proxy(None)
         self.set_audio_format(AUDIO_FORMAT)
         
-    def xmlrpc_call(self, method, args=[], url_args=True, secure=False):
+    def xmlrpc_call(self, method, args=[], url_args=True, secure=False, includeTime=True):
         if url_args is True:
             url_args = args
             
         args = args[:]
-        args.insert(0, int(time.time()))
+        
+        if includeTime:
+            args.insert(0, int(time.time()+self.time_offset))
+            
         if self.authToken:
             args.insert(1, self.authToken)
             
@@ -88,7 +93,8 @@ class Pandora(object):
         data = pandora_encrypt(xml)
         
         url_arg_strings = []
-        if self.rid:
+        
+        if self.rid and includeTime:
             url_arg_strings.append('rid=%s'%self.rid)
         if self.listenerId:
             url_arg_strings.append('lid=%s'%self.listenerId)
@@ -116,9 +122,9 @@ class Pandora(object):
         except urllib2.URLError as e:
             logging.error("Network error: %s", e)
             if e.reason[0] == 'timed out':
-                raise PandoraTimeout("Network Timeout")
+                raise PandoraTimeout("Network error", submsg="Timeout")
             else:
-                raise PandoraNetError("Network error: %s"%e.reason[1])
+                raise PandoraNetError("Network error", submsg=e.reason[1])
             
         logging.debug(text)
        
@@ -132,8 +138,13 @@ class Pandora(object):
                 raise PandoraAuthTokenInvalid(msg)
             elif code == 'INCOMPATIBLE_VERSION':
                 raise PandoraAPIVersionError(msg)
+            elif code == 'OUT_OF_SYNC':
+                raise PandoraError("Out of sync", code,
+                    submsg="Correct your system's clock. If the problem persists, a Pithos update may be required")
+            elif code == 'AUTH_INVALID_USERNAME_PASSWORD':
+                raise PandoraError("Login Error", code, submsg="Invalid username or password")
             else:
-                raise PandoraError(msg, code)
+                raise PandoraError("Pandora returned an error", code, "%s: %s"%(code, msg))
         else:
             return xmlrpc_parse(tree)
 
@@ -150,6 +161,10 @@ class Pandora(object):
     def connect(self, user, password):
         self.rid = "%07iP"%(int(time.time()) % 10000000)
         self.listenerId = self.authToken = None
+        
+        pandora_time = self.xmlrpc_call('misc.sync', [], [], secure=True, includeTime=False)
+        pandora_time = int(re.sub(r"\D", "", pandora_decrypt(pandora_time)))
+        self.time_offset =  pandora_time - time.time()
             
         user = self.xmlrpc_call('listener.authenticateListener', [user, password], [], secure=True)
         
